@@ -3,13 +3,14 @@ from typing import Dict, List, Tuple, Union
 import isodate
 import timely_beliefs as tb
 from sqlalchemy.orm import Query
-from marshmallow import ValidationError, validate, validates, fields, validates_schema
 
 from flexmeasures.data.config import db
-from flexmeasures.data import ma
-from flexmeasures.data.models.time_series import Sensor, SensorSchemaMixin, TimedValue
-from flexmeasures.data.models.markets import Market
 from flexmeasures.data.models.user import User
+from flexmeasures.data.models.time_series import Sensor, TimedValue
+from flexmeasures.data.models.generic_assets import (
+    create_generic_asset,
+    GenericAssetType,
+)
 from flexmeasures.utils.entity_address_utils import build_entity_address
 from flexmeasures.utils.flexmeasures_inflection import humanize, pluralize
 
@@ -31,6 +32,10 @@ class AssetType(db.Model):
     yearly_seasonality = db.Column(db.Boolean(), nullable=False, default=False)
 
     def __init__(self, **kwargs):
+        generic_asset_type = GenericAssetType(
+            name=kwargs["name"], description=kwargs.get("hover_label", None)
+        )
+        db.session.add(generic_asset_type)
         super(AssetType, self).__init__(**kwargs)
         self.name = self.name.replace(" ", "_").lower()
         if "display_name" not in kwargs:
@@ -105,17 +110,27 @@ class Asset(db.Model, tb.SensorDBMixin):
     def __init__(self, **kwargs):
 
         # Create a new Sensor with unique id across assets, markets and weather sensors
+        # Also keep track of ownership.
         if "id" not in kwargs:
-            new_sensor = Sensor(name=kwargs["name"])
+            generic_assets_arg = kwargs.copy()
+            if "owner_id" in kwargs:
+                owner = User.query.get(kwargs["owner_id"])
+                if owner:
+                    generic_assets_arg.update(account_id=owner.account_id)
+            new_generic_asset = create_generic_asset("asset", **generic_assets_arg)
+            new_sensor = Sensor(name=kwargs["name"], generic_asset=new_generic_asset)
             db.session.add(new_sensor)
             db.session.flush()  # generates the pkey for new_sensor
             sensor_id = new_sensor.id
         else:
             # The UI may initialize Asset objects from API form data with a known id
             sensor_id = kwargs["id"]
-
+        if "unit" not in kwargs:
+            kwargs["unit"] = "MW"  # current default
         super(Asset, self).__init__(**kwargs)
         self.id = sensor_id
+        if self.unit != "MW":
+            raise Exception("FlexMeasures only supports MW as unit for now.")
         self.name = self.name.replace(" (MW)", "")
         if "display_name" not in kwargs:
             self.display_name = humanize(self.name)
@@ -135,11 +150,19 @@ class Asset(db.Model, tb.SensorDBMixin):
         return self.unit
 
     @property
-    def entity_address(self) -> str:
+    def entity_address_fm0(self) -> str:
+        """Entity address under the fm0 scheme for entity addresses."""
         return build_entity_address(
-            dict(owner_id=self.owner_id, asset_id=self.id), "connection"
+            dict(owner_id=self.owner_id, asset_id=self.id),
+            "connection",
+            fm_scheme="fm0",
         )
 
+    @property
+    def entity_address(self) -> str:
+        """Entity address under the latest fm scheme for entity addresses."""
+        return build_entity_address(dict(sensor_id=self.id), "sensor")
+
     @property
     def location(self) -> Tuple[float, float]:
         return self.latitude, self.longitude
@@ -179,67 +202,6 @@ class Asset(db.Model, tb.SensorDBMixin):
         )
 
 
-class AssetSchema(SensorSchemaMixin, ma.SQLAlchemySchema):
-    """
-    Asset schema, with validations.
-    """
-
-    class Meta:
-        model = Asset
-
-    @validates("name")
-    def validate_name(self, name: str):
-        asset = Asset.query.filter(Asset.name == name).one_or_none()
-        if asset:
-            raise ValidationError(f"An asset with the name {name} already exists.")
-
-    @validates("owner_id")
-    def validate_owner(self, owner_id: int):
-        owner = User.query.get(owner_id)
-        if not owner:
-            raise ValidationError(f"Owner with id {owner_id} doesn't exist.")
-        if "Prosumer" not in owner.flexmeasures_roles:
-            raise ValidationError(
-                "Asset owner must have role 'Prosumer'."
-                f" User {owner_id} has roles {[r.name for r in owner.flexmeasures_roles]}."
-            )
-
-    @validates("market_id")
-    def validate_market(self, market_id: int):
-        market = Market.query.get(market_id)
-        if not market:
-            raise ValidationError(f"Market with id {market_id} doesn't exist.")
-
-    @validates("asset_type_name")
-    def validate_asset_type(self, asset_type_name: str):
-        asset_type = AssetType.query.get(asset_type_name)
-        if not asset_type:
-            raise ValidationError(f"Asset type {asset_type_name} doesn't exist.")
-
-    @validates_schema(skip_on_field_errors=False)
-    def validate_soc_constraints(self, data, **kwargs):
-        if "max_soc_in_mwh" in data and "min_soc_in_mwh" in data:
-            if data["max_soc_in_mwh"] < data["min_soc_in_mwh"]:
-                errors = {
-                    "max_soc_in_mwh": "This value must be equal or higher than the minimum soc."
-                }
-                raise ValidationError(errors)
-
-    id = ma.auto_field()
-    display_name = fields.Str(validate=validate.Length(min=4))
-    capacity_in_mw = fields.Float(required=True, validate=validate.Range(min=0.0001))
-    min_soc_in_mwh = fields.Float(validate=validate.Range(min=0))
-    max_soc_in_mwh = fields.Float(validate=validate.Range(min=0))
-    soc_in_mwh = ma.auto_field()
-    soc_datetime = ma.auto_field()
-    soc_udi_event_id = ma.auto_field()
-    latitude = fields.Float(required=True, validate=validate.Range(min=-90, max=90))
-    longitude = fields.Float(required=True, validate=validate.Range(min=-180, max=180))
-    asset_type_name = ma.auto_field(required=True)
-    owner_id = ma.auto_field(required=True)
-    market_id = ma.auto_field(required=True)
-
-
 def assets_share_location(assets: List[Asset]) -> bool:
     """
     Return True if all assets in this list are located on the same spot.
@@ -294,7 +256,7 @@ class Power(TimedValue, db.Model):
         super(Power, self).__init__(**kwargs)
 
     def __repr__(self):
-        return "<Power %.2f on Asset %s at %s by DataSource %s, horizon %s>" % (
+        return "<Power %.5f on Asset %s at %s by DataSource %s, horizon %s>" % (
             self.value,
             self.asset_id,
             self.datetime,
