@@ -6,10 +6,8 @@ from sqlalchemy.ext.declarative import declared_attr
 from sqlalchemy.orm import Query, Session
 import timely_beliefs as tb
 import timely_beliefs.utils as tb_utils
-from marshmallow import Schema, fields
 
 from flexmeasures.data.config import db
-from flexmeasures.data import ma
 from flexmeasures.data.queries.utils import (
     add_belief_timing_filter,
     add_user_source_filter,
@@ -18,7 +16,9 @@ from flexmeasures.data.queries.utils import (
     exclude_source_type_filter,
 )
 from flexmeasures.data.services.time_series import collect_time_series_data
+from flexmeasures.utils.entity_address_utils import build_entity_address
 from flexmeasures.data.models.charts import chart_type_to_chart_specs
+from flexmeasures.data.models.generic_assets import GenericAsset
 from flexmeasures.utils.time_utils import server_now
 from flexmeasures.utils.flexmeasures_inflection import capitalize
 
@@ -26,11 +26,29 @@ from flexmeasures.utils.flexmeasures_inflection import capitalize
 class Sensor(db.Model, tb.SensorDBMixin):
     """A sensor measures events. """
 
-    def __init__(self, name: str, **kwargs):
+    generic_asset_id = db.Column(
+        db.Integer,
+        db.ForeignKey("generic_asset.id", ondelete="CASCADE"),
+        nullable=False,
+    )
+    generic_asset = db.relationship(
+        "GenericAsset",
+        foreign_keys=[generic_asset_id],
+        backref=db.backref(
+            "sensors", lazy=True, cascade="all, delete-orphan", passive_deletes=True
+        ),
+    )
+
+    def __init__(self, name: str, generic_asset: GenericAsset, **kwargs):
         tb.SensorDBMixin.__init__(self, name, **kwargs)
         tb_utils.remove_class_init_kwargs(tb.SensorDBMixin, kwargs)
+        kwargs["generic_asset"] = generic_asset
         db.Model.__init__(self, **kwargs)
 
+    @property
+    def entity_address(self) -> str:
+        return build_entity_address(dict(sensor_id=self.id), "sensor")
+
     def search_beliefs(
         self,
         event_starts_after: Optional[datetime_type] = None,
@@ -118,7 +136,7 @@ class Sensor(db.Model, tb.SensorDBMixin):
 
     @property
     def timerange(self) -> Dict[str, datetime_type]:
-        """Timerange for which sensor data exists.
+        """Time range for which sensor data exists.
 
         :returns: dictionary with start and end, for example:
                   {
@@ -145,7 +163,7 @@ class Sensor(db.Model, tb.SensorDBMixin):
         return dict(start=least_recent.event_start, end=most_recent.event_end)
 
     def __repr__(self) -> str:
-        return f"<Sensor {self.id}: {self.name}>"
+        return f"<Sensor {self.id}: {self.name}, unit: {self.unit} res.: {self.event_resolution}>"
 
 
 class TimedBelief(db.Model, tb.TimedBeliefDBMixin):
@@ -240,35 +258,6 @@ class TimedBelief(db.Model, tb.TimedBeliefDBMixin):
         return tb.TimedBelief.__repr__(self)
 
 
-class SensorSchemaMixin(Schema):
-    """
-    Base sensor schema.
-
-    Here we include all fields which are implemented by timely_beliefs.SensorDBMixin
-    All classes inheriting from timely beliefs sensor don't need to repeat these.
-    In a while, this schema can represent our unified Sensor class.
-
-    When subclassing, also subclass from `ma.SQLAlchemySchema` and add your own DB model class, e.g.:
-
-        class Meta:
-            model = Asset
-    """
-
-    name = ma.auto_field(required=True)
-    unit = ma.auto_field(required=True)
-    timezone = ma.auto_field()
-    event_resolution = fields.TimeDelta(required=True, precision="minutes")
-
-
-class SensorSchema(SensorSchemaMixin, ma.SQLAlchemySchema):
-    """
-    Sensor schema, with validations.
-    """
-
-    class Meta:
-        model = Sensor
-
-
 class TimedValue(object):
     """
     A mixin of all tables that store time series data, either forecasts or measurements.
