@@ -1,7 +1,7 @@
 """CLI Tasks for (de)populating the database - most useful in development"""
 
 from datetime import timedelta
-from typing import List, Optional
+from typing import Dict, List, Optional
 
 import pandas as pd
 import pytz
@@ -16,10 +16,19 @@ import timely_beliefs as tb
 from flexmeasures.data import db
 from flexmeasures.data.services.forecasting import create_forecasting_jobs
 from flexmeasures.data.services.users import create_user
-from flexmeasures.data.models.time_series import Sensor, SensorSchema, TimedBelief
-from flexmeasures.data.models.assets import Asset, AssetSchema
+from flexmeasures.data.models.user import Account, AccountRole, RolesAccounts
+from flexmeasures.data.models.time_series import Sensor, TimedBelief
+from flexmeasures.data.schemas.sensors import SensorSchema
+from flexmeasures.data.schemas.generic_assets import (
+    GenericAssetSchema,
+    GenericAssetTypeSchema,
+)
+from flexmeasures.data.models.assets import Asset
+from flexmeasures.data.schemas.assets import AssetSchema
+from flexmeasures.data.models.generic_assets import GenericAsset, GenericAssetType
 from flexmeasures.data.models.markets import Market
-from flexmeasures.data.models.weather import WeatherSensor, WeatherSensorSchema
+from flexmeasures.data.models.weather import WeatherSensor
+from flexmeasures.data.schemas.weather import WeatherSensorSchema
 from flexmeasures.data.models.data_sources import (
     get_or_create_source,
     get_source_or_none,
@@ -37,17 +46,65 @@ def fm_dev_add_data():
     """Developer CLI commands not yet meant for users: Add data."""
 
 
+@fm_add_data.command("account-role")
+@with_appcontext
+@click.option("--name", required=True)
+@click.option("--description")
+def new_account_role(name: str, description: str):
+    """
+    Create an account role.
+    """
+    role = AccountRole.query.filter_by(name=name).one_or_none()
+    if role is not None:
+        click.echo(f"Account role '{name}' already exists.")
+        raise click.Abort
+    role = AccountRole(name=name, description=description)
+    db.session.add(role)
+    db.session.commit()
+    print(f"Account role '{name}' (ID: {role.id}) successfully created.")
+
+
+@fm_add_data.command("account")
+@with_appcontext
+@click.option("--name", required=True)
+@click.option("--roles", help="e.g. anonymous,Prosumer,CPO")
+def new_account(name: str, roles: List[str]):
+    """
+    Create an account for a tenant in the FlexMeasures platform.
+    """
+    account = db.session.query(Account).filter_by(name=name).one_or_none()
+    if account is not None:
+        click.echo(f"Account '{name}' already exists.")
+        raise click.Abort
+    account = Account(name=name)
+    db.session.add(account)
+    if roles:
+        for role_name in roles.split(","):
+            role = AccountRole.query.filter_by(name=role_name).one_or_none()
+            if role is None:
+                print(f"Adding account role {role_name} ...")
+                role = AccountRole(name=role_name)
+                db.session.add(role)
+            db.session.flush()
+            db.session.add(RolesAccounts(role_id=role.id, account_id=account.id))
+    db.session.commit()
+    print(f"Account '{name}' (ID: {account.id}) successfully created.")
+
+
 @fm_add_data.command("user")
 @with_appcontext
 @click.option("--username", required=True)
 @click.option("--email", required=True)
+@click.option("--account-id", type=int, required=True)
 @click.option("--roles", help="e.g. anonymous,Prosumer,CPO")
 @click.option(
     "--timezone",
     default="UTC",
     help="timezone as string, e.g. 'UTC' or 'Europe/Amsterdam'",
 )
-def new_user(username: str, email: str, roles: List[str], timezone: str):
+def new_user(
+    username: str, email: str, account_id: int, roles: List[str], timezone: str
+):
     """
     Create a FlexMeasures user.
 
@@ -57,7 +114,11 @@ def new_user(username: str, email: str, roles: List[str], timezone: str):
     try:
         pytz.timezone(timezone)
     except pytz.UnknownTimeZoneError:
-        print("Timezone %s is unknown!" % timezone)
+        print(f"Timezone {timezone} is unknown!")
+        raise click.Abort
+    account = db.session.query(Account).get(account_id)
+    if account is None:
+        print(f"No account with id {account_id} found!")
         raise click.Abort
     pwd1 = getpass.getpass(prompt="Please enter the password:")
     pwd2 = getpass.getpass(prompt="Please repeat the password:")
@@ -68,11 +129,12 @@ def new_user(username: str, email: str, roles: List[str], timezone: str):
         username=username,
         email=email,
         password=hash_password(pwd1),
+        account_name=account.name,
         timezone=timezone,
         user_roles=roles,
-        check_deliverability=False,
+        check_email_deliverability=False,
     )
-    app.db.session.commit()
+    db.session.commit()
     print(f"Successfully created user {created_user}")
 
 
@@ -91,25 +153,89 @@ def new_user(username: str, email: str, roles: List[str], timezone: str):
     required=True,
     help="timezone as string, e.g. 'UTC' or 'Europe/Amsterdam'",
 )
+@click.option(
+    "--generic-asset-id",
+    required=True,
+    type=int,
+    help="Generic asset to assign this sensor to",
+)
 def add_sensor(**args):
     """Add a sensor."""
     check_timezone(args["timezone"])
     check_errors(SensorSchema().validate(args))
     args["event_resolution"] = timedelta(minutes=args["event_resolution"])
     sensor = Sensor(**args)
-    app.db.session.add(sensor)
-    app.db.session.commit()
+    db.session.add(sensor)
+    db.session.commit()
     print(f"Successfully created sensor with ID {sensor.id}")
-    # TODO: uncomment when #66 has landed
-    # print(f"You can access it at its entity address {sensor.entity_address}")
+    print(f"You can access it at its entity address {sensor.entity_address}")
+
+
+@fm_dev_add_data.command("generic-asset-type")
+@with_appcontext
+@click.option("--name", required=True)
+@click.option(
+    "--hover-label",
+    type=str,
+    help="Label visible when hovering over the name in the UI.\n"
+    "Useful to explain acronyms, for example.",
+)
+def add_generic_asset_type(**args):
+    """Add a generic asset type."""
+    check_errors(GenericAssetTypeSchema().validate(args))
+    generic_asset_type = GenericAssetType(**args)
+    db.session.add(generic_asset_type)
+    db.session.commit()
+    print(f"Successfully created generic asset type with ID {generic_asset_type.id}")
+    print("You can now assign generic assets to it")
+
+
+@fm_dev_add_data.command("generic-asset")
+@with_appcontext
+@click.option("--name", required=True)
+@click.option(
+    "--latitude",
+    type=float,
+    help="Latitude of the asset's location",
+)
+@click.option(
+    "--longitude",
+    type=float,
+    help="Longitude of the asset's location",
+)
+@click.option("--account-id", type=int, required=True)
+@click.option(
+    "--generic-asset-type-id",
+    required=True,
+    type=int,
+    help="Generic asset type to assign to this asset",
+)
+def add_generic_asset(**args):
+    """Add a generic asset."""
+    check_errors(GenericAssetSchema().validate(args))
+    generic_asset = GenericAsset(**args)
+    db.session.add(generic_asset)
+    db.session.commit()
+    print(f"Successfully created generic asset with ID {generic_asset.id}")
+    print("You can now assign sensors to it")
 
 
 @fm_add_data.command("asset")
 @with_appcontext
 @click.option("--name", required=True)
 @click.option("--asset-type-name", required=True)
-@click.option("--unit", required=True, help="e.g. MW, kW/h", default="MW")
-@click.option("--capacity-in-MW", required=True, type=float)
+@click.option(
+    "--unit",
+    help="unit of rate, just MW (default) for now",
+    type=click.Choice(["MW"]),
+    default="MW",
+)  # TODO: enable others
+@click.option(
+    "--capacity-in-MW",
+    required=True,
+    type=float,
+    help="Maximum rate of this asset in MW",
+)
 @click.option(
     "--event-resolution",
     required=True,
@@ -158,8 +284,8 @@ def new_asset(**args):
     check_errors(AssetSchema().validate(args))
     args["event_resolution"] = timedelta(minutes=args["event_resolution"])
     asset = Asset(**args)
-    app.db.session.add(asset)
-    app.db.session.commit()
+    db.session.add(asset)
+    db.session.commit()
     print(f"Successfully created asset with ID {asset.id}")
     print(f"You can access it at its entity address {asset.entity_address}")
 
@@ -198,11 +324,10 @@ def add_weather_sensor(**args):
     check_errors(WeatherSensorSchema().validate(args))
     args["event_resolution"] = timedelta(minutes=args["event_resolution"])
     sensor = WeatherSensor(**args)
-    app.db.session.add(sensor)
-    app.db.session.commit()
-    print(f"Successfully created sensor with ID {sensor.id}")
-    # TODO: uncomment when #66 has landed
-    # print(f"You can access it at its entity address {sensor.entity_address}")
+    db.session.add(sensor)
+    db.session.commit()
+    print(f"Successfully created weather sensor with ID {sensor.id}")
+    print(f" You can access it at its entity address {sensor.entity_address}")
 
 
 @fm_add_data.command("structure")
@@ -211,7 +336,7 @@ def add_initial_structure():
     """Initialize structural data like asset types, market types and weather sensor types."""
     from flexmeasures.data.scripts.data_gen import populate_structure
 
-    populate_structure(app.db)
+    populate_structure(db)
 
 
 @fm_dev_add_data.command("beliefs")
@@ -274,6 +399,27 @@ def add_initial_structure():
     type=int,
     help="Column number with values (1 is 2nd column, the default)",
 )
+@click.option(
+    "--delimiter",
+    required=True,
+    type=str,
+    default=",",
+    help="[For csv files] Character to delimit columns per row, defaults to comma",
+)
+@click.option(
+    "--decimal",
+    required=False,
+    default=".",
+    type=str,
+    help="[For csv files] decimal character, e.g. '.' for 10.5",
+)
+@click.option(
+    "--thousands",
+    required=False,
+    default="",
+    type=str,
+    help="[For csv files] thousands separator, e.g. '.' for 10.035,2",
+)
 @click.option(
     "--sheet_number",
     required=False,
@@ -291,7 +437,11 @@ def add_beliefs(
     nrows: Optional[int] = None,
     datecol: int = 0,
     valuecol: int = 1,
+    delimiter: str = ",",
+    decimal: str = ".",
+    thousands: str = "",
     sheet_number: Optional[int] = None,
+    **kwargs,  # in-code calls to this CLI command can set additional kwargs for use in pandas.read_csv or pandas.read_excel
 ):
     """Add sensor data from a csv file (also accepts xls or xlsx).
 
@@ -324,9 +474,11 @@ def add_beliefs(
         _source = get_or_create_source(source, source_type="CLI script")
 
     # Set up optional parameters for read_csv
-    kwargs = dict()
     if file.split(".")[-1].lower() == "csv":
         kwargs["infer_datetime_format"] = True
+        kwargs["delimiter"] = delimiter
+        kwargs["decimal"] = decimal
+        kwargs["thousands"] = thousands
     if sheet_number is not None:
         kwargs["sheet_name"] = sheet_number
     if horizon is not None:
@@ -442,23 +594,24 @@ def create_forecasts(
                 asset_id=asset_id,
                 timed_value_type=value_type,
                 horizons=[horizon],
-                start_of_roll=from_date - timedelta(hours=horizon),
-                end_of_roll=to_date - timedelta(hours=horizon),
+                start_of_roll=from_date - horizon,
+                end_of_roll=to_date - horizon,
             )
     else:
         from flexmeasures.data.scripts.data_gen import populate_time_series_forecasts
 
         populate_time_series_forecasts(
-            app.db, horizons, from_date, to_date, asset_type, asset_id
+            db, horizons, from_date, to_date, asset_type, asset_id
         )
 
 
 @fm_add_data.command("external-weather-forecasts")
+@with_appcontext
 @click.option(
     "--region",
     type=str,
     default="",
-    help="Name of the region (will create sub-folder, should later tag the forecast in the DB, probably).",
+    help="Name of the region (will create sub-folder if you store json files, should later probably tag the forecast in the DB).",
 )
 @click.option(
     "--location",
@@ -473,7 +626,7 @@ def create_forecasts(
     "--num_cells",
     type=int,
     default=1,
-    help="Number of cells on the grid. Only used if a region of interest has been mapped in the location parameter.",
+    help="Number of cells on the grid. Only used if a region of interest has been mapped in the location parameter. Defaults to 1.",
 )
 @click.option(
     "--method",
@@ -484,13 +637,13 @@ def create_forecasts(
 @click.option(
     "--store-in-db/--store-as-json-files",
     default=False,
-    help="Store forecasts in the database, or simply save as json files.",
+    help="Store forecasts in the database, or simply save as json files. (defaults to json files)",
 )
 def collect_weather_data(region, location, num_cells, method, store_in_db):
     """
-    Collect weather forecasts from the DarkSky API
+    Collect weather forecasts from the OpenWeatherMap API
 
-    This function can get weather data for one location or for several location within
+    This function can get weather data for one location or for several locations within
     a geometrical grid (See the --location parameter).
     """
     from flexmeasures.data.scripts.grid_weather import get_weather_forecasts
@@ -510,7 +663,7 @@ def check_timezone(timezone):
         raise click.Abort
 
 
-def check_errors(errors: list):
+def check_errors(errors: Dict[str, List[str]]):
     if errors:
         print(
             f"Please correct the following errors:\n{errors}.\n Use the --help flag to learn more."
